# -*- coding:Utf-8 -*-

from __future__ import with_statement

import es
import os
import urllib
import cfglib
import zipfile
import usermsg
import playerlib
import installlib
import gamethread

	
info          = es.AddonInfo()
info.name     = 'Chatbox Titles'
info.version  = '1.2'
info.basename = 'chatbox'
info.author   = 'FMIS @K Menace.'
info.url      = 'http://addons.eventscripts.com/addons/view/47120'

CHATBOX = installlib.getInstaller('chatboxtitles')
	
CONFIGPATH = (es.getAddonPath(info.basename) + '/configuration.ini').replace('\\','/')
if not os.path.isfile(CONFIGPATH) and CHATBOX:
	CHATBOX.update(True)

CONFIG = cfglib.AddonINI(CONFIGPATH)

TITRES           = CONFIG['Titres']
ANTIFLOOD        = bool(CONFIG['Configuration']['AntiFlood'])
ADDCOMMAND       = CONFIG['Configuration']['AddCommand']
REMCOMMAND       = CONFIG['Configuration']['RemCommand']
ADMINGROUPS      = CONFIG['Configuration']['AdminGroups']
FILTERCHARS      = CONFIG['Configuration']['FilterChars']
SHOWDEADMSG      = bool(CONFIG['Configuration']['ShowDeadMsg'])
ANTIFLOODTIME    = float(CONFIG['Configuration']['AntiFloodTime'])
FILTERCOMMANDS   = CONFIG['Configuration']['FilterCommands']
DELETEASCIICHARS = bool(CONFIG['Configuration']['DeleteASCIIChars'])


PLAYERTEAMS = {1:'spec', 2:'t', 3:'ct'}

SCRIPTVAR = es.ServerVar('chatbox_version', info.version, 'Version of the Chatbox Titles\'s script by FMIS @k Menace.')
SCRIPTVAR.makepublic()

class TitleManager(dict):
	def addPlayer(self, steamid, title):
		if not steamid in self:
			if title in TITRES:
				CONFIG.addValueToGroup('Titres', title, TITRES[title] + ', ' + steamid, overwrite=True)
			else:
				CONFIG.addValueToGroup('Titres', title, steamid, True)
			self[steamid] = title
			CONFIG.write()
			
	def removePlayer(self, steamid):
		if steamid in self:
			CONFIG.addValueToGroup('Titres', self[steamid], TITRES[self[steamid]].replace(steamid + ', ', '').replace(', ' + steamid, '').replace(steamid, ''), overwrite=True)
			del self[steamid]
			CONFIG.write()
		
	def getPlayersTitle(self):
		for title in TITRES:
			for steamid in TITRES[title].split(', '):
				self[steamid] = title

class FloodManager(list):
	def addPlayer(self, userid):
		if userid not in self:
			self.append(userid)
			gamethread.delayed(ANTIFLOODTIME, self.removePlayer, userid)

	def removePlayer(self, userid):
		if userid in self:
			self.remove(userid)
			
def sayFilter(userid, text, team):
	userid, text, team = int(userid), text.strip('"'), bool(team)
	player = playerlib.getPlayer(userid)
	if es.exists('saycommand', text.split()[0]) or text[0] in FILTERCHARS or text.split()[0] in FILTERCOMMANDS.split(', ') or not player.steamid in tm:
		return (userid, text, team)
	if ANTIFLOOD:
		if userid in fm:
			return ('', 0, '')
		fm.addPlayer(userid)
	if DELETEASCIICHARS:
		for char in map(lambda x: chr(x), range(0, 32)):
			if char in text:
				text = text.replace(char, '')
	target  = '#human' if not team else '#' + PLAYERTEAMS[player.team]
	message = '\5[\4%s\5] \3%s \1: %s' % (tm[player.steamid], player.name, text)
	if player.isdead:
		if not SHOWDEADMSG:
			target += ',#dead'
		message = '\3*\1DEAD\3* ' + message
	for uid in playerlib.getUseridList(target):
		usermsg.saytext2(uid, player.index, message)
	return ('', 0, '')
		
def addPlayerTitle():
	args              = es.getargs().split()
	userid, userid2   = es.getuserid(args[0]), es.getcmduserid()
	steamid, steamid2 = es.getplayersteamid(userid), es.getplayersteamid(userid2)
	if not es.exists('userid', userid):
		es.tell(userid2, "Cannot found userid for '" + args[0] + "' sorry.")
		return
	if not steamid2 in tm:
		es.tell(userid2, "You are not titled, so, you can't access to this command")
		return
	if not len(args) >= 2:
		es.tell(userid2, "Syntax : " + ADDCOMMAND + " <userid|part of name|steamid> <title>")
		return
	if not tm[steamid2] in ADMINGROUPS:
		es.tell(userid2, "You're not authorized to use this command!")
		return
	if steamid in tm:
		tm.removePlayer(steamid)
	tm.addPlayer(steamid, args[len(args)-1].replace('_', ''))

def removePlayerTitle():
	args              = es.getargs().split()
	userid, userid2   = es.getuserid(args[0]), es.getcmduserid()
	steamid, steamid2 = es.getplayersteamid(userid), es.getplayersteamid(userid2)
	if not es.exists('userid', userid):
		es.tell(userid2, "Cannot found userid for '" + args[0] + "', sorry.")
		return
	if not steamid2 in tm:
		es.tell(userid2, "You are not titled, so, you can't access to this command")
		return
	if not len(args) >= 1:
		es.tell(userid2, "Syntax : " + ADDCOMMAND + " <userid|part of name|steamid> <title>")
		return
	if not tm[steamid2] in ADMINGROUPS:
		es.tell(userid2, "You're not authorized to use this command!")
		return
	if not steamid in tm:
		es.tell(userid2, "Player '" + es.getplayername(userid) + "' have no title.")
		return
	tm.removePlayer(steamid)
	
def es_map_start(ev):
	if CHATBOX:
		CHATBOX.update()
	tm.getPlayersTitle()
	
def load(): 
	if CHATBOX:
		CHATBOX.update()
	es.addons.registerSayFilter(sayFilter)
	es.regsaycmd(ADDCOMMAND, info.basename + '/addPlayerTitle')
	es.regsaycmd(REMCOMMAND, info.basename + '/removePlayerTitle')
	tm.getPlayersTitle()
	
def unload():
	es.addons.unregisterSayFilter(sayFilter)
	es.unregsaycmd(ADDCOMMAND)
	es.unregsaycmd(REMCOMMAND)
	
tm = TitleManager()
fm = FloodManager()